@@ -1,13 +1,14 @@
 from typing import Callable
-from fastapi import Depends
+
+from fastapi import Depends, HTTPException
 from pydantic import BaseModel
 
-from . import CRUDGenerator, NOT_FOUND
-
+from . import CRUDGenerator, NOT_FOUND, utils
 
 try:
     from sqlalchemy.orm import Session
     from sqlalchemy.ext.declarative import DeclarativeMeta
+    from sqlalchemy.exc import IntegrityError
 except ImportError:
     sqlalchemy_installed = False
     Session = None
@@ -23,13 +24,14 @@ class SQLAlchemyCRUDRouter(CRUDGenerator):
 
         self.db_model = db_model
         self.db_func = db
-        self._primary_key: str = db_model.__table__.primary_key.columns.keys()[0]
+        self._pk: str = db_model.__table__.primary_key.columns.keys()[0]
+        self._pk_type: type = utils.get_pk_type(schema, self._pk)
 
         if 'prefix' not in kwargs:
             kwargs['prefix'] = db_model.__tablename__
 
         if 'create_schema' not in kwargs:
-            kwargs['create_schema'] = self.schema_factory(schema, self._primary_key)
+            kwargs['create_schema'] = self.schema_factory(schema, self._pk)
 
         super().__init__(schema, *args, **kwargs)
 
@@ -40,7 +42,7 @@ class SQLAlchemyCRUDRouter(CRUDGenerator):
         return route
 
     def _get_one(self) -> Callable:
-        def route(item_id, db: Session = Depends(self.db_func)):
+        def route(item_id: self._pk_type, db: Session = Depends(self.db_func)):
             model = db.query(self.db_model).get(item_id)
 
             if model:
@@ -52,20 +54,23 @@ class SQLAlchemyCRUDRouter(CRUDGenerator):
 
     def _create(self) -> Callable:
         def route(model: self.create_schema, db: Session = Depends(self.db_func)):
-            db_model = self.db_model(**model.dict())
-            db.add(db_model)
-            db.commit()
-            db.refresh(db_model)
-
-            return db_model
+            try:
+                db_model = self.db_model(**model.dict())
+                db.add(db_model)
+                db.commit()
+                db.refresh(db_model)
+                return db_model
+            except IntegrityError:
+                db.rollback()
+                raise HTTPException(422, 'Key already exists')
 
         return route
 
     def _update(self) -> Callable:
-        def route(item_id: int, model: self.schema, db: Session = Depends(self.db_func)):
+        def route(item_id: self._pk_type, model: self.schema, db: Session = Depends(self.db_func)):
             db_model = self._get_one()(item_id, db)
 
-            for key, value in model.dict(exclude={self._primary_key}).items():
+            for key, value in model.dict(exclude={self._pk}).items():
                 if hasattr(db_model, key):
                     setattr(db_model, key, value)
 
@@ -86,11 +91,11 @@ class SQLAlchemyCRUDRouter(CRUDGenerator):
         return route
 
     def _delete_one(self) -> Callable:
-        def route(item_id: int, db: Session = Depends(self.db_func)):
+        def route(item_id: self._pk_type, db: Session = Depends(self.db_func)):
             db_model = self._get_one()(item_id, db)
             db.delete(db_model)
             db.commit()
 
             return db_model
 
-        return route
\ No newline at end of file
+        return route
